@@ -4,18 +4,24 @@ import numpy as np
 import requests, json
 
 class quantim:
-    def __init__(self, username, password, secretpool, env="qa"):
+    def __init__(self, username, password, secretpool, env="pdn", api_url=None):
         self.username = username
         self.password = password
         self.secretpool = secretpool
         self.env = env
 
+        if api_url is None:
+            self.api_url = "https://api-quantimqa.sura-im.com/" if env=="qa" else "https://api-quantim.sura-im.com/"
+        else:
+            self.api_url = api_url
+
     def get_header(self):
+
         if self.secretpool=='ALM':
-            token_url = "https://api-quantimqa.sura-im.com/oauthback/token" if self.env=="qa" else "https://api-quantim.sura-im.com/oauthback/token"
+            token_url = f"{self.api_url}token"
             data = {"username":self.username, "password":self.password}
         else:
-            token_url = "https://api-quantimqa.sura-im.com/tokendynamicpool" if self.env=="qa" else "https://api-quantim.sura-im.com/tokendynamicpool"
+            token_url = f"{self.api_url}tokendynamicpool"
             data = {"username":self.username, "password":self.password, "secretpool":self.secretpool}
 
         headers = {"Accept": "*/*",'Content-Type': 'application/json', 'Access-Control-Allow-Origin': '*'}
@@ -31,18 +37,22 @@ class quantim:
         data: when method get, data is an array of key values.
         '''
         api_call_headers = self.get_header()
+        api_url = f"{self.api_url}{endpoint}"
         if method.lower()=='post':
-            api_url = f"{'https://api-quantimqa.sura-im.com/' if self.env=='qa' else 'https://api-quantim.sura-im.com/'}{endpoint}"
             api_call_response = requests.post(api_url, headers=api_call_headers, data=json.dumps(data), verify=verify)
         elif method.lower()=='get':
-            api_url = f"{'https://api-quantimqa.sura-im.com/' if self.env=='qa' else 'https://api-quantim.sura-im.com/'}{endpoint}"
             if data is not None:
                 api_url = api_url + '?'+'&'.join([f"{x['key']}={x['value']}" for x in data])
             api_call_response = requests.get(api_url, headers=api_call_headers, data=None, verify=verify)
         else:
             print("Method not supported!")
             return None
-        return json.loads(api_call_response.text)
+        
+        try:
+            resp = json.loads(api_call_response.text)
+        except:
+            resp = api_call_response.text
+        return resp
 
     def retrieve_s3_df(self, bucket, key, sep=','):
         '''
@@ -64,3 +74,27 @@ class quantim:
         except:
             resp = {'success':False, 'message':'Check permissions!'}
         return resp
+
+    def upload_with_presigned_url(local_file_path, bucket, key):
+        """
+        Upload a local file to S3 using a presigned URL.
+        """
+        data = {'bucket':bucket, 'key':key}
+        try:
+            presigned_url = self.api_call('link_data_s3', method="post", data=data, verify=False)
+        except Exception as e:
+            print(f"Error with presigned url: {e}")
+            return False
+
+        with open(local_file_path, 'rb') as file:
+            try:
+                response = requests.put(presigned_url, data=file, verify=False)
+                if response.status_code == 200:
+                    print("File uploaded successfully")
+                    return True
+                else:
+                    print(f"Error uploading file. Status code: {response.status_code}")
+                    return False
+            except Exception as e:
+                print(f"Error uploading file: {e}")
+                return False
